@@ -42,8 +42,7 @@ import (
 	"github.com/sirupsen/logrus"
 	cdi "tags.cncf.io/container-device-interface/pkg/parser"
 
-	"github.com/docker/compose/v2/pkg/api"
-	"github.com/docker/compose/v2/pkg/progress"
+	"github.com/docker/compose/v5/pkg/api"
 )
 
 type createOptions struct {
@@ -61,9 +60,9 @@ type createConfigs struct {
 }
 
 func (s *composeService) Create(ctx context.Context, project *types.Project, createOpts api.CreateOptions) error {
-	return progress.RunWithTitle(ctx, func(ctx context.Context) error {
+	return Run(ctx, func(ctx context.Context) error {
 		return s.create(ctx, project, createOpts)
-	}, s.stdinfo(), "Creating")
+	}, "create", s.events)
 }
 
 func (s *composeService) create(ctx context.Context, project *types.Project, options api.CreateOptions) error {
@@ -93,7 +92,7 @@ func (s *composeService) create(ctx context.Context, project *types.Project, opt
 		return err
 	}
 
-	volumes, err := s.ensureProjectVolumes(ctx, project, options.AssumeYes)
+	volumes, err := s.ensureProjectVolumes(ctx, project)
 	if err != nil {
 		return err
 	}
@@ -150,13 +149,13 @@ func (s *composeService) ensureNetworks(ctx context.Context, project *types.Proj
 	return networks, nil
 }
 
-func (s *composeService) ensureProjectVolumes(ctx context.Context, project *types.Project, assumeYes bool) (map[string]string, error) {
+func (s *composeService) ensureProjectVolumes(ctx context.Context, project *types.Project) (map[string]string, error) {
 	ids := map[string]string{}
 	for k, volume := range project.Volumes {
 		volume.CustomLabels = volume.CustomLabels.Add(api.VolumeLabel, k)
 		volume.CustomLabels = volume.CustomLabels.Add(api.ProjectLabel, project.Name)
 		volume.CustomLabels = volume.CustomLabels.Add(api.VersionLabel, api.ComposeVersion)
-		id, err := s.ensureVolume(ctx, k, volume, project, assumeYes)
+		id, err := s.ensureVolume(ctx, k, volume, project)
 		if err != nil {
 			return nil, err
 		}
@@ -211,11 +210,17 @@ func (s *composeService) getCreateConfigs(ctx context.Context,
 	if err != nil {
 		return createConfigs{}, err
 	}
+
+	exposed, err := buildContainerPorts(service)
+	if err != nil {
+		return createConfigs{}, err
+	}
+
 	containerConfig := container.Config{
 		Hostname:        service.Hostname,
 		Domainname:      service.DomainName,
 		User:            service.User,
-		ExposedPorts:    buildContainerPorts(service),
+		ExposedPorts:    exposed,
 		Tty:             tty,
 		OpenStdin:       stdinOpen,
 		StdinOnce:       opts.AttachStdin && stdinOpen,
@@ -764,17 +769,24 @@ func setBlkio(blkio *types.BlkioConfig, resources *container.Resources) {
 	}
 }
 
-func buildContainerPorts(s types.ServiceConfig) nat.PortSet {
+func buildContainerPorts(s types.ServiceConfig) (nat.PortSet, error) {
 	ports := nat.PortSet{}
 	for _, s := range s.Expose {
-		p := nat.Port(s)
-		ports[p] = struct{}{}
+		proto, port := nat.SplitProtoPort(s)
+		start, end, err := nat.ParsePortRange(port)
+		if err != nil {
+			return nil, err
+		}
+		for i := start; i <= end; i++ {
+			p := nat.Port(fmt.Sprintf("%d/%s", i, proto))
+			ports[p] = struct{}{}
+		}
 	}
 	for _, p := range s.Ports {
 		p := nat.Port(fmt.Sprintf("%d/%s", p.Target, p.Protocol))
 		ports[p] = struct{}{}
 	}
-	return ports
+	return ports, nil
 }
 
 func buildContainerPortBindingOptions(s types.ServiceConfig) nat.PortMap {
@@ -900,7 +912,7 @@ func bindRequiresMountAPI(bind *types.ServiceVolumeBind) bool {
 	switch {
 	case bind == nil:
 		return false
-	case !bind.CreateHostPath:
+	case !bool(bind.CreateHostPath):
 		return true
 	case bind.Propagation != "":
 		return true
@@ -1211,7 +1223,7 @@ func buildBindOption(bind *types.ServiceVolumeBind) *mount.BindOptions {
 	}
 	opts := &mount.BindOptions{
 		Propagation:      mount.Propagation(bind.Propagation),
-		CreateMountpoint: bind.CreateHostPath,
+		CreateMountpoint: bool(bind.CreateHostPath),
 	}
 	switch bind.Recursive {
 	case "disabled":
@@ -1394,15 +1406,14 @@ func (s *composeService) resolveOrCreateNetwork(ctx context.Context, project *ty
 	}
 
 	networkEventName := fmt.Sprintf("Network %s", n.Name)
-	w := progress.ContextWriter(ctx)
-	w.Event(progress.CreatingEvent(networkEventName))
+	s.events.On(creatingEvent(networkEventName))
 
 	resp, err := s.apiClient().NetworkCreate(ctx, n.Name, createOpts)
 	if err != nil {
-		w.Event(progress.ErrorEvent(networkEventName))
+		s.events.On(errorEvent(networkEventName, err.Error()))
 		return "", fmt.Errorf("failed to create network %s: %w", n.Name, err)
 	}
-	w.Event(progress.CreatedEvent(networkEventName))
+	s.events.On(createdEvent(networkEventName))
 
 	err = s.connectNetwork(ctx, n.Name, dangledContainers, nil)
 	if err != nil {
@@ -1444,17 +1455,17 @@ func (s *composeService) removeDivergedNetwork(ctx context.Context, project *typ
 
 	err = s.apiClient().NetworkRemove(ctx, n.Name)
 	eventName := fmt.Sprintf("Network %s", n.Name)
-	progress.ContextWriter(ctx).Event(progress.RemovedEvent(eventName))
+	s.events.On(removedEvent(eventName))
 	return containers, err
 }
 
 func (s *composeService) disconnectNetwork(
 	ctx context.Context,
-	network string,
+	nwName string,
 	containers Containers,
 ) error {
 	for _, c := range containers {
-		err := s.apiClient().NetworkDisconnect(ctx, network, c.ID, true)
+		err := s.apiClient().NetworkDisconnect(ctx, nwName, c.ID, true)
 		if err != nil {
 			return err
 		}
@@ -1465,12 +1476,12 @@ func (s *composeService) disconnectNetwork(
 
 func (s *composeService) connectNetwork(
 	ctx context.Context,
-	network string,
+	nwName string,
 	containers Containers,
 	config *network.EndpointSettings,
 ) error {
 	for _, c := range containers {
-		err := s.apiClient().NetworkConnect(ctx, network, c.ID, config)
+		err := s.apiClient().NetworkConnect(ctx, nwName, c.ID, config)
 		if err != nil {
 			return err
 		}
@@ -1530,7 +1541,7 @@ func (s *composeService) resolveExternalNetwork(ctx context.Context, n *types.Ne
 	}
 }
 
-func (s *composeService) ensureVolume(ctx context.Context, name string, volume types.VolumeConfig, project *types.Project, assumeYes bool) (string, error) {
+func (s *composeService) ensureVolume(ctx context.Context, name string, volume types.VolumeConfig, project *types.Project) (string, error) {
 	inspected, err := s.apiClient().VolumeInspect(ctx, volume.Name)
 	if err != nil {
 		if !errdefs.IsNotFound(err) {
@@ -1562,13 +1573,10 @@ func (s *composeService) ensureVolume(ctx context.Context, name string, volume t
 	}
 	actual, ok := inspected.Labels[api.ConfigHashLabel]
 	if ok && actual != expected {
-		confirm := assumeYes
-		if !assumeYes {
-			msg := fmt.Sprintf("Volume %q exists but doesn't match configuration in compose file. Recreate (data will be lost)?", volume.Name)
-			confirm, err = s.prompt(msg, false)
-			if err != nil {
-				return "", err
-			}
+		msg := fmt.Sprintf("Volume %q exists but doesn't match configuration in compose file. Recreate (data will be lost)?", volume.Name)
+		confirm, err := s.prompt(msg, false)
+		if err != nil {
+			return "", err
 		}
 		if confirm {
 			err = s.removeDivergedVolume(ctx, name, volume, project)
@@ -1623,8 +1631,7 @@ func (s *composeService) removeDivergedVolume(ctx context.Context, name string,
 
 func (s *composeService) createVolume(ctx context.Context, volume types.VolumeConfig) error {
 	eventName := fmt.Sprintf("Volume %s", volume.Name)
-	w := progress.ContextWriter(ctx)
-	w.Event(progress.CreatingEvent(eventName))
+	s.events.On(creatingEvent(eventName))
 	hash, err := VolumeHash(volume)
 	if err != nil {
 		return err
@@ -1637,9 +1644,9 @@ func (s *composeService) createVolume(ctx context.Context, volume types.VolumeCo
 		DriverOpts: volume.DriverOpts,
 	})
 	if err != nil {
-		w.Event(progress.ErrorEvent(eventName))
+		s.events.On(errorEvent(eventName, err.Error()))
 		return err
 	}
-	w.Event(progress.CreatedEvent(eventName))
+	s.events.On(createdEvent(eventName))
 	return nil
 }
