@@ -29,14 +29,6 @@ import (
 	gsync "sync"
 	"time"
 
-	pathutil "github.com/docker/compose/v2/internal/paths"
-	"github.com/docker/compose/v2/internal/sync"
-	"github.com/docker/compose/v2/internal/tracing"
-	"github.com/docker/compose/v2/pkg/api"
-	"github.com/docker/compose/v2/pkg/progress"
-	cutils "github.com/docker/compose/v2/pkg/utils"
-	"github.com/docker/compose/v2/pkg/watch"
-
 	"github.com/compose-spec/compose-go/v2/types"
 	"github.com/compose-spec/compose-go/v2/utils"
 	ccli "github.com/docker/cli/cli/command/container"
@@ -44,8 +36,16 @@ import (
 	"github.com/docker/docker/api/types/filters"
 	"github.com/docker/docker/api/types/image"
 	"github.com/go-viper/mapstructure/v2"
+	"github.com/moby/buildkit/util/progress/progressui"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sync/errgroup"
+
+	pathutil "github.com/docker/compose/v5/internal/paths"
+	"github.com/docker/compose/v5/internal/sync"
+	"github.com/docker/compose/v5/internal/tracing"
+	"github.com/docker/compose/v5/pkg/api"
+	cutils "github.com/docker/compose/v5/pkg/utils"
+	"github.com/docker/compose/v5/pkg/watch"
 )
 
 type WatchFunc func(ctx context.Context, project *types.Project, options api.WatchOptions) (func() error, error)
@@ -89,6 +89,9 @@ func (w *Watcher) Start(ctx context.Context) error {
 	w.stopFn = cancelFunc
 	wait, err := w.watchFn(ctx, w.project, w.options)
 	if err != nil {
+		go func() {
+			w.errCh <- err
+		}()
 		return err
 	}
 	go func() {
@@ -469,7 +472,7 @@ func (t tarDockerClient) Exec(ctx context.Context, containerID string, cmd []str
 		})
 	}
 	eg.Go(func() error {
-		_, err := io.Copy(t.s.stdinfo(), conn.Reader)
+		_, err := io.Copy(t.s.stdout(), conn.Reader)
 		return err
 	})
 
@@ -594,6 +597,7 @@ func (s *composeService) exec(ctx context.Context, project *types.Project, servi
 			exec.Privileged = x.Privileged
 			exec.Command = x.Command
 			exec.Workdir = x.WorkingDir
+			exec.DetachKeys = s.configFile().DetachKeys
 			for _, v := range x.Environment.ToMapping().Values() {
 				err := exec.Env.Set(v)
 				if err != nil {
@@ -610,7 +614,7 @@ func (s *composeService) rebuild(ctx context.Context, project *types.Project, se
 	options.LogTo.Log(api.WatchLogger, fmt.Sprintf("Rebuilding service(s) %q after changes were detected...", services))
 	// restrict the build to ONLY this service, not any of its dependencies
 	options.Build.Services = services
-	options.Build.Progress = progress.ModePlain
+	options.Build.Progress = string(progressui.PlainMode)
 	options.Build.Out = cutils.GetWriter(func(line string) {
 		options.LogTo.Log(api.WatchLogger, line)
 	})
